@@ -167,7 +167,7 @@ except NameError:
 
 __author__ = 'Joseph Kocherhans, Jacob Kaplan-Moss, Daniel Lindsley'
 __all__ = ['Solr']
-__version__ = (2, 0, 13)
+__version__ = (2, 0, 14)
 
 def get_version():
     return "%s.%s.%s" % __version__[:3]
@@ -251,7 +251,7 @@ class Solr(object):
         if len(netloc) == 1:
             self.host, self.port = netloc[0], None
         else:
-            self.host, self.port = netloc
+            self.host, self.port = netloc[0], int(netloc[1])
         self.path = path.rstrip('/')
         self.timeout = timeout
         self.log = self._get_log()
@@ -328,7 +328,7 @@ class Solr(object):
         path = '%s/terms/?%s' % (self.path, safe_urlencode(params, True))
         return self._send_request('GET', path)
     
-    def _update(self, message, clean_ctrl_chars=True, commit=True):
+    def _update(self, message, clean_ctrl_chars=True, commit=True, waitFlush=None, waitSearcher=None):
         """
         Posts the given xml message to http://<host>:<port>/solr/update and
         returns the result.
@@ -343,8 +343,16 @@ class Solr(object):
         # Per http://wiki.apache.org/solr/UpdateXmlMessages, we can append a
         # ``commit=true`` to the URL and have the commit happen without a
         # second request.
-        if commit:
-            path += '?commit=true'
+        query_vars = []
+        if commit is not None:
+            query_vars.append('commit=%s' % str(bool(commit)).lower())
+        if waitFlush is not None:
+            query_vars.append('waitFlush=%s' % str(bool(waitFlush)).lower())
+        if waitSearcher is not None:
+            query_vars.append('waitSearcher=%s' % str(bool(waitSearcher)).lower())
+        if query_vars:
+            path = '%s?%s' % (path, '&'.join(query_vars))
+            
         
         # Clean the message of ctrl characters.
         if clean_ctrl_chars:
@@ -375,7 +383,7 @@ class Solr(object):
         """
         # identify the responding server
         server_type = None
-        server_string = headers.get('server', None)
+        server_string = headers.get('server', '')
         
         if 'jetty' in server_string.lower():
             server_type = 'jetty'
@@ -587,14 +595,15 @@ class Solr(object):
         self.log.debug("Found '%d' Term suggestions results.", sum(len(j) for i, j in res.items()))
         return res
     
-    def add(self, docs, commit=True, boost=None):
+    def add(self, docs, commit=True, boost=None, commitWithin=None, waitFlush=None, waitSearcher=None):
         """Adds or updates documents. For now, docs is a list of dictionaries
         where each key is the field name and each value is the value to index.
         """
         start_time = time.time()
         self.log.debug("Starting to build add request...")
         message = ET.Element('add')
-        
+        if commitWithin:
+            message.set('commitWithin', commitWithin)
         for doc in docs:
             d = ET.Element('doc')
             
@@ -637,12 +646,12 @@ class Solr(object):
             
             message.append(d)
         
-        m = ET.tostring(message, 'utf-8')
+        m = ET.tostring(message, encoding='utf-8')
         end_time = time.time()
         self.log.debug("Built add request of %s docs in %0.2f seconds." % (len(docs), end_time - start_time))
-        response = self._update(m, commit=commit)
+        response = self._update(m, commit=commit, waitFlush=waitFlush, waitSearcher=waitSearcher)
     
-    def delete(self, id=None, q=None, commit=True, fromPending=True, fromCommitted=True):
+    def delete(self, id=None, q=None, commit=True, waitFlush=None, waitSearcher=None):
         """Deletes documents."""
         if id is None and q is None:
             raise ValueError('You must specify "id" or "q".')
@@ -653,13 +662,21 @@ class Solr(object):
         elif q is not None:
             m = '<delete><query>%s</query></delete>' % q
         
-        response = self._update(m, commit=commit)
+        response = self._update(m, commit=commit, waitFlush=waitFlush, waitSearcher=waitSearcher)
     
-    def commit(self):
-        response = self._update('<commit />')
+    def commit(self, waitFlush=None, waitSearcher=None, expungeDeletes=None):
+        if expungeDeletes is not None:
+            msg = '<commit expungeDeletes="%s" />' % str(bool(expungeDeletes)).lower()
+        else:
+            msg = '<commit />'
+        response = self._update(msg, waitFlush=waitFlush, waitSearcher=waitSearcher)
     
-    def optimize(self):
-        response = self._update('<optimize />')
+    def optimize(self, waitFlush=None, waitSearcher=None, maxSegments=None):
+        if maxSegments:
+            msg = '<commit maxSegments="%d" />' % maxSegments
+        else:
+            msg = '<commit />'
+        response = self._update('<optimize />', waitFlush=waitFlush, waitSearcher=waitSearcher)
 
 
 class SolrCoreAdmin(object):
